@@ -25,14 +25,14 @@
 #include "sch_easyeda_parser.h"
 #include "sch_io_easyeda.h"
 
-#include <schematic.h>
-#include <sch_sheet.h>
-#include <sch_screen.h>
+#include <font/fontconfig.h>
 #include <kiplatform/environment.h>
 #include <project_sch.h>
-#include <wildcards_and_files_ext.h>
+#include <sch_screen.h>
+#include <sch_sheet.h>
+#include <schematic.h>
 #include <string_utils.h>
-
+#include <wildcards_and_files_ext.h>
 #include <wx/log.h>
 #include <wx/stdstream.h>
 #include <wx/zipstrm.h>
@@ -225,7 +225,19 @@ LIB_SYMBOL* loadSymbol( const wxString& aLibraryPath, nlohmann::json aFileData,
                         {
                             parts.RemoveAt( 0 );
 
-                            return parser.ParseSymbol( origin, paramMap, parts );
+                            LIB_SYMBOL* ksymbol = parser.ParseSymbol( origin, paramMap, parts );
+
+                            // Clear reference numbers
+                            LIB_FIELD& refField = ksymbol->GetReferenceField();
+                            wxString   origRef = refField.GetText();
+                            wxString   reference;
+
+                            for( size_t i = 0; i < origRef.size() && !wxIsdigit( origRef[i] ); i++ )
+                                reference << origRef[i];
+
+                            refField.SetText( reference );
+
+                            return ksymbol;
                         }
                     }
                 }
@@ -261,7 +273,19 @@ LIB_SYMBOL* loadSymbol( const wxString& aLibraryPath, nlohmann::json aFileData,
 
             VECTOR2D origin( topDoc.head.x, topDoc.head.y );
 
-            return parser.ParseSymbol( origin, *c_para, topDoc.shape );
+            LIB_SYMBOL* ksymbol = parser.ParseSymbol( origin, *c_para, topDoc.shape );
+
+            // Clear reference numbers
+            LIB_FIELD& refField = ksymbol->GetReferenceField();
+            wxString   origRef = refField.GetText();
+            wxString   reference;
+
+            for( size_t i = 0; i < origRef.size() && !wxIsdigit( origRef[i] ); i++ )
+                reference << origRef[i];
+
+            refField.SetText( reference );
+
+            return ksymbol;
         }
     }
     catch( nlohmann::json::exception& e )
@@ -285,6 +309,9 @@ void SCH_IO_EASYEDA::EnumerateSymbolLib( wxArrayString&         aSymbolNameList,
 {
     std::map<wxString, int> namesCounter;
 
+    // Suppress font substitution warnings
+    fontconfig::FONTCONFIG::SetReporter( nullptr );
+
     try
     {
         wxFFileInputStream in( aLibraryPath );
@@ -588,6 +615,9 @@ SCH_SHEET* SCH_IO_EASYEDA::LoadSchematicFile( const wxString& aFileName, SCHEMAT
 {
     wxCHECK( !aFileName.IsEmpty() && aSchematic, nullptr );
 
+    // Show the font substitution warnings
+    fontconfig::FONTCONFIG::SetReporter( &WXLOG_REPORTER::GetInstance() );
+
     SCH_SHEET* rootSheet = nullptr;
 
     if( aAppendToMe )
