@@ -511,15 +511,6 @@ void PANEL_FP_EDITOR_DEFAULTS::OnDeleteTextItem( wxCommandEvent& event )
     if( selectedRows.empty() )
         return;
 
-    for( int row : selectedRows )
-    {
-        if( row < 2 )
-        {
-            DisplayError( nullptr, _( "Reference and value are mandatory." ) );
-            return;
-        }
-    }
-
     if( !m_textItemsGrid->CommitPendingChanges() || !m_graphicsGrid->CommitPendingChanges() )
         return;
 
@@ -548,5 +539,3 @@ void PANEL_FP_EDITOR_DEFAULTS::ResetPanel()
 
     loadFPSettings( &cfg );
 }
-
-
